@@ -6,7 +6,7 @@ use syn::spanned::Spanned;
 use syn::{BinOp, Expr, Pat, PathSegment, Result, Stmt, UnOp};
 
 use crate::common;
-use crate::common::{squash, TS};
+use crate::common::{squash, DFFSetupArgs, TS};
 
 pub(crate) fn hdl_gen_process(item: syn::ItemFn) -> Result<TS> {
     let signature = &item.sig;
@@ -601,6 +601,67 @@ fn hdl_macro(x: &syn::ExprMacro) -> Result<TS> {
                 .replace("\")", "");
             Ok(quote!(ast::VerilogStatement::Comment(#invocation_as_string.to_string())))
         }
+        "dff_setup" => {
+            let args: DFFSetupArgs = x.mac.parse_body()?;
+            let args_clock = &args.clock;
+            let args_reset = &args.reset;
+            let clk = common::fixup_ident(quote!(#args_clock).to_string());
+            let reset = common::fixup_ident(quote!(#args_reset).to_string());
+            let dffs_clk = &args
+                .dffs
+                .iter()
+                .map(|x| common::fixup_ident(quote!(#x.clock.next).to_string()))
+                .collect::<Vec<_>>();
+            let dffs_reset = &args
+                .dffs
+                .iter()
+                .map(|x| common::fixup_ident(quote!(#x.reset.next).to_string()))
+                .collect::<Vec<_>>();
+            let dffs_d = &args
+                .dffs
+                .iter()
+                .map(|x| common::fixup_ident(quote!(#x.d.next).to_string()))
+                .collect::<Vec<_>>();
+            let dffs_q = &args
+                .dffs
+                .iter()
+                .map(|x| common::fixup_ident(quote!(#x.q).to_string()))
+                .collect::<Vec<_>>();
+            Ok(quote!(
+                {
+                    let mut ret = vec![];
+                    #(ret.push(ast::VerilogStatement::Assignment(ast::VerilogExpression::Signal(#dffs_clk.to_string()), ast::VerilogExpression::Signal(#clk.to_string()))));*;
+                    #(ret.push(ast::VerilogStatement::Assignment(ast::VerilogExpression::Signal(#dffs_reset.to_string()), ast::VerilogExpression::Signal(#reset.to_string()))));*;
+                    #(ret.push(ast::VerilogStatement::Assignment(ast::VerilogExpression::Signal(#dffs_d.to_string()), ast::VerilogExpression::Signal(#dffs_q.to_string()))));*;
+                    ast::VerilogStatement::Macro(ret)
+                }
+            ))
+        }
+        "clock_reset" => {
+            let args: DFFSetupArgs = x.mac.parse_body()?;
+            let args_clock = &args.clock;
+            let args_reset = &args.reset;
+            let clk = common::fixup_ident(quote!(#args_clock).to_string());
+            let reset = common::fixup_ident(quote!(#args_reset).to_string());
+            let dffs_clk = &args
+                .dffs
+                .iter()
+                .map(|x| common::fixup_ident(quote!(#x.clock.next).to_string()))
+                .collect::<Vec<_>>();
+            let dffs_reset = &args
+                .dffs
+                .iter()
+                .map(|x| common::fixup_ident(quote!(#x.reset.next).to_string()))
+                .collect::<Vec<_>>();
+            Ok(quote!(
+                {
+                    let mut ret = vec![];
+                    #(ret.push(ast::VerilogStatement::Assignment(ast::VerilogExpression::Signal(#dffs_clk.to_string()), ast::VerilogExpression::Signal(#clk.to_string()))));*;
+                    #(ret.push(ast::VerilogStatement::Assignment(ast::VerilogExpression::Signal(#dffs_reset.to_string()), ast::VerilogExpression::Signal(#reset.to_string()))));*;
+                    ast::VerilogStatement::Macro(ret)
+                }
+            ))
+        }
         _ => Err(syn::Error::new(
             x.span(),
             "Unsupported macro invocation in HDL",
